package com.interfaceware.chameleon;

import java.text.SimpleDateFormat;
/**
* This class handles Date Time objects.
* @see com.interfaceware.chameleon.DblDate
**/
public class ChameleonDateTime
{
   private long m_DateTimeHandle;
   private boolean m_IsOwner;

   private static ChameleonDateTime m_NowDate = null;

   static
   {
      System.loadLibrary("chm_java");
   }   
/**
* Construct a ChameleonDateTime object.
**/
   public ChameleonDateTime() throws ChameleonException
   {
      m_DateTimeHandle = CHMdateTimeCreate();
      m_IsOwner = true;
      CHMdateTimeAddRef(m_DateTimeHandle);
   }
/**
* Construct a ChameleonDateTime object with the specified long Date Time handle.
* This constructor is used in the internal implementation of Chameleon and should not be invoked directly.
**/
   public ChameleonDateTime(long Handle) throws ChameleonException
   {
      m_DateTimeHandle = Handle;
      m_IsOwner = false;
   }

/**
* Get the current Date Time to the nearest millisecond of the system clock.
* @see com.interfaceware.chameleon.DblDate#now()
**/
   public static ChameleonDateTime now() throws ChameleonException
   {
      if (m_NowDate == null)
      {
         m_NowDate = new ChameleonDateTime();
      }
      m_NowDate.setValue(DblDate.now());
      return m_NowDate;
	}

/**
* Free the ChameleonDateTime object handle.
**/
   public void release() throws ChameleonException
   {
      if ((0 != m_DateTimeHandle) && m_IsOwner)
      {
         CHMdateTimeRelease(m_DateTimeHandle);
         m_DateTimeHandle = 0;
      }
   }
/**
* Finalizer that invokes the release method.
**/
   protected void finalize() throws ChameleonException
   {
      release();
   }
/**
* Construct a ChameleonDateTime object with the specified DblDate.
**/
   public ChameleonDateTime(DblDate Value) throws ChameleonException
   {
      m_DateTimeHandle = CHMdateTimeCreate();
      m_IsOwner = true;
      CHMdateTimeAddRef(m_DateTimeHandle);
      setValue(Value);
   }

/**
* Construct a ChameleonDateTime object by parsing a string to produce a date.
* <br>The date format of the string should be set before calling this constructor by setting
* the chameleon.date.format system property.  
* <br>For example,
* <pre>
* 	System.setProperty(chameleon.date.format, "yyyyMMdd");
* 	ChameleonDateTime dt = new ChameleonDateTime("20010203");
* </pre>
* Or set the date format in the call to java:
* <pre>
* 	> java -Dchameleon.date.format=yyyyMMdd
* </pre> 
* @see java.text.DateFormat#parse(java.lang.String)
**/
   public ChameleonDateTime(String dateString) throws ChameleonException 
   {
      this();

      try 
      {
         String dateFormatString = System.getProperty("chameleon.date.format");
         if(dateFormatString != null) 
         {
            setValue(new DblDate( 
               new SimpleDateFormat( dateFormatString).parse(dateString)));
         } 
         else 
         {
            throw new ChameleonException("chameleon.date.format not set!");
         }
      } 
      catch (java.text.ParseException pe) 
      {
         throw new ChameleonException("date parsing error: " + pe.getMessage());
      }
   }

/**
* Get the Date Time value.
**/
   public DblDate value() throws ChameleonException
   {
      return new DblDate(CHMdateTimeGetDATE(m_DateTimeHandle));
   }

/**
* Set the Date Time value with the specified DblDate.
**/
   public void setValue(DblDate Value) throws ChameleonException 
   {
      CHMdateTimeSetDATE(m_DateTimeHandle, Value.toDouble());
   }

/**
* Get the time zone as a short integer.
* For example, for a ChameleonDateTime with a time in the EST this method would return -5.
**/
   public short zone() throws ChameleonException
   {
      return CHMdateTimeGetZone(m_DateTimeHandle); 
   }

/**
* Set the time zone with the specified short integer.
* For example, pass in -5 for a time in the EST.
**/
   public void setZone(short Zone) throws ChameleonException
   {
      CHMdateTimeSetZone(m_DateTimeHandle, Zone);
   }
/**
* Check whether the ChameleonDateTime object is empty.
* For example, if the Date Time is equal to "" then this method will
* return true.
**/
   public boolean isNull() throws ChameleonException
   {
      return CHMdateTimeIsNull(m_DateTimeHandle);
   }
/**
* Return the date as a string using the format string passed
* according to the formatting rules for the ANSI C <code>strftime</code> function.
* For example, if you want 19/01/2007 to be formatted as Jan. 19, 2007, you would use
* "%b. %d, %Y" for <code>FormatString</code>.
**/
   public String format(String FormatString) throws ChameleonException
   {
      return CHMdateTimeFormat(m_DateTimeHandle, FormatString);
   }

/**
* Convert the Date Time value of the ChameleonDateTime object to a String.
**/
   public String toString()
   {
      try
      {
         if (isNull())
         {
            return "<null>";
         }
         else
         {
            return value().toString();
         }
      }
      catch (ChameleonException Error)
      {
         return Error.toString() + " in ChameleonDateTime.toString()";       
      }
   }

/**
* Get the Date Time handle for the class object.
* This is used in the internal implementation of Chameleon and should not be called.
**/
   protected long handle()
   {
      return m_DateTimeHandle;
   }


   private native long CHMdateTimeCreate() throws ChameleonException;
   private native void CHMdateTimeAddRef    (long DateHandle) throws
   ChameleonException;
   private native void CHMdateTimeRelease   (long DateHandle) throws
   ChameleonException;
   private native double CHMdateTimeGetDATE (long DateHandle) throws
   ChameleonException;
   private native double CHMdateTimeGetSecondFraction(long DateHandle) throws
   ChameleonException;
   private native short CHMdateTimeGetZone(long DateHandle) throws
   ChameleonException;
   private native void CHMdateTimeSetDATE
   (
      long    DateHandle,
      double Value
   ) throws ChameleonException;

   private native void CHMdateTimeSetSecondFraction
   (
      long    DateHandle,
      double NewValue
   ) throws ChameleonException;

   private native void CHMdateTimeSetZone
   (
      long   DateHandle,
      short NewValue
   ) throws ChameleonException;

   private native boolean CHMdateTimeIsNull
   (
      long     DateHandle
   ) throws ChameleonException;

   private native String CHMdateTimeFormat
   (
      long    DateHandle,
      String FormatString
   ) throws ChameleonException;
}


