package com.interfaceware.chameleon;

import java.io.*;
import java.net.*;
import java.util.*;

/*=================================================================
   Connection class
   - handles communication with the client
=================================================================*/
/**
* This class implements a connection for streaming data to and from a
* client socket.
* @see com.interfaceware.chameleon.Transport
**/
public class Connection extends Hl7ClientSocket implements Transport
{
   protected Hl7ClientSocket m_Socket = null;
   protected Wrapper    m_Wrapper = null;
   protected Log        m_Log = null;
   protected ServerSocketListener  m_Listener = null;

   protected static final int EOF = -1;

/**
 * default constructor
 **/
   public Connection(){};
   
   static
   {
      System.loadLibrary("chm_java");
   }
/**
* Constructs a Connection with the specified socket and wrapper objects,
* and the log interface.
* The constructor initialises the input/output data streams for the socket
* and starts the thread.
* The wrapper is used to wrap and unwrap message data passed through
* the data streams. Without a wrapper, messages cannot be sent or received.
* When the log interface is null, error messages are written to the console.
**/
   public Connection(long TransportId, Wrapper AWrapper, Log ALog)

   {
      super( TransportId );
      m_Wrapper = AWrapper;
      m_Log = ALog;
      clientConnectEvent();
   }

/**
* Listens on the client socket port for incoming messages.
* The incoming data is added to a buffer and the buffer is sent to
* the wrapper. A valid wrapper object must be passed in the constructor,
* in order to unwrap the incoming message.
* @see com.interfaceware.chameleon.Wrapper
**/
   public void onMessage( long TransportId, String Message )
   {
     StringBuffer MessageAsBuffer = new StringBuffer( Message );
     m_Wrapper.checkForMessage( MessageAsBuffer, this );
   }


/**
* Wraps a message and writes the message to the socket output stream.
* A valid wrapper object must be passed in the constructor, in order to
* wrap the outgoing message.
**/
   public void sendMessage(String MessageData)
   {
      try
      {
         super.sendMessage( MessageData );
      }
      catch( ChameleonException Error )
      {
         errorEvent(Error,"write error");
      }
   }

   /**
   * called when an error occurs
   **/
   public void onTransportError( long TransportId, int ErrorId, String ErrorMessage )
   {
      if (m_Log != null)
      {
         m_Log.println(ErrorMessage);        
      }
      else
      {
         System.out.println(ErrorMessage);
      }
   }


/**
* Converts the Exception to a string and appends a description to the
* error message. The error message is output using the log interface passed
* to the constructor.
**/
   protected void errorEvent(Exception Error, String Description)
   {
      if (null != m_Log)
      {
         m_Log.println(Error.toString() + " : " + Description);
      }
      else
      {
         System.out.println(Error.toString() + " : " + Description);
      }
   }

   private void getDataWithoutNullChars(byte[] Buffer, int CountOfCharacter)
   {
      for (int CharIndex = 0; CharIndex < CountOfCharacter; CharIndex += 1)
      {
         if (0 == Buffer[CharIndex])
         {
            Buffer[CharIndex] = 32;
         }
      }
   }

   /**
    * Logs line to the log object assigned to this class
    **/

   protected void println(String Description)
   {
      if (null != m_Log)
      {
         m_Log.println(Description);
      }
      else
      {
         System.out.println(Description);
      }
   }

   /**
   * called when a connection closes
   **/
   public void onClose( long TransportId )
   {
      if (m_Listener != null)
      {
         m_Listener.onClientDisconnect( this );            
      }
   }
   /**

   /**
    * Invokes the onClientConnect event on the ServerSocketListener class
    **/
   protected void clientConnectEvent()
   {
      if (m_Listener != null)
      {
         m_Listener.onClientConnect(m_Socket);
      }
   }

/**
Adds the specified server socket listener to receive server socket events
from this connection.
**/
   public void addServerSocketListener(ServerSocketListener AListener)
   {
      m_Listener = AListener;
   }
}
