package com.interfaceware.chameleon;

/**
* This class handles Chameleon Engine objects.
**/
abstract public class Engine
{
   long m_EngineHandle;
   private long m_EventHandle;
   private Log  m_Log;
/**
* Construct an Engine object.
**/
   public Engine() throws ChameleonException
   {
      m_EngineHandle = CHMengineCreate();
      m_EventHandle = CHMengineAddListener(m_EngineHandle);
      CHMengineAddRef(m_EngineHandle);
   }

/**
* Release the Engine object handle.
**/
   public void release() throws ChameleonException
   {
      if (0 != m_EngineHandle)
      {
         CHMengineRelease(m_EngineHandle, m_EventHandle);
         m_Log = null;
         m_EngineHandle = 0;
      }
   }   
/**
* Finalizer that invokes the release method.
**/
   protected void finalize() throws ChameleonException
   {
      release();
   }

   static
   {
      System.loadLibrary("chm_java");
   }
/**
* Load the visual message definition (VMD) file <code>MappingFile</code>.
* This must be done before the Engine can be used.
* Applications should load the VMD file as part of initialization since
* loading may take a few seconds.  Loading the VMD file every time a message
* is encountered will result in very slow performance.
**/
   public synchronized void load(String MappingFile) throws ChameleonException
   {
      CHMengineLoad(m_EngineHandle, MappingFile);
   }


   private void log(String Line)
   {
      // This call back gives us the log output from the python log statement in Java
      // println will only output stuff if we have a log object assigned to this engine.
      println(Line);
   }

/**
* Parse flatwire messages into a table if stubcode is being used.  It must be overridden by descendant
* classes.  It can also be overridden for logging purposes, or any other application where the user would need to
* access the flatwire format of the message before parsing it.
* Because the Transport interface is no longer being used (see {@link Transport Transport} for more information),
* a null object can be passed in for the Transport argument.
**/
   public void parseMessage(String Flatwire, Transport ATransport) throws ChameleonException {}


   /**
   *  Translate a HL7 message into XML.  The format of the XML output conforms to the DTDs 
   *  of the format specified in the XML tab 
   *  of the Options window in the Chameleon GUI for your VMD.
   * For example code, see the XML example in the New Project
   * wizard.
   * @see <a href="http://www.interfaceware.com/manual/xml.html">http://www.interfaceware.com/manual/xml.html</a>
   **/
   public String translateMessageToXml(String Flatwire) throws ChameleonException
   {
	   return CHMengineTranslateMessageToXml(m_EngineHandle, Flatwire);
   }

   /**
   *  Translate XML encoded HL7 into a HL7 message.  The format of the XML message should conform to the DTDs 
   *  of the format specified in the XML tab of the Options window in the Chameleon GUI for your VMD.
   * For example code, see the XML example in the New Project
   * wizard.
   * @see <a href="http://www.interfaceware.com/manual/xml.html">http://www.interfaceware.com/manual/xml.html</a>
   **/
   public String translateXmlToMessage(String Xml) throws ChameleonException
   { 
	   return CHMengineTranslateXmlToMessage(m_EngineHandle, Xml);
   }

/**
* Call the native C DLL method to parse the specified flatwire string
* into a Table.  Returns the message index.
**/
   protected int parse(String Flatwire, Table ResultTable) throws ChameleonException
   {
       return CHMengineParseMessage(m_EngineHandle,
                                    Flatwire,
                                    ResultTable.handle());
   }

/**
* Set the option to enforce strict segment grammar in a parsed message.
* The default value is false.  When this flag is set the parser will check for the presence of
* compulsory fields in each segment and enforce other rules such as maximum field length.
* This option can also be set in the Misc tab of the Options window in the Chameleon GUI.
**/
   public void setRejectBadSegmentGrammar(boolean Value) throws ChameleonException
   {
      CHMengineSetRejectBadSegmentGrammarFlag(m_EngineHandle,
                                              Value);
   }

/**
* Get a message string in a printable format by appending a newline character '\n'
* after every carriage return character '\r'.
**/
   public String makeMessagePrintable(String Message) throws ChameleonException
   {
      return CHMengineMakeMessagePrintable(m_EngineHandle, Message);   
   }

/**
* Get the number of messages that are defined in the currently loaded VMD file.
* Usage of this method with {@link #messageName(int) messageName} 
* offers an easy way to iterate through the messages.
**/
   public int countOfMessage() throws ChameleonException
   {
      return CHMengineCountOfMessage(m_EngineHandle);
   }

/**
 * Get the count of identifiers for a given message.  
 * Usage of this method with {@link #messageIdentity(int, int) messageIdentity}
 * offers an easy way to iterate through each message to
 * get the regular expression used for matching for
 * each message.
**/
   public int countOfMessageIdentifier(int MessageIndex) throws ChameleonException
   {
      return CHMengineCountOfMessageIdentifier(m_EngineHandle , MessageIndex);
   }

/**
 * Get the regular expression used to match a field in a message's identity
 * list.  Usage of this method with {@link #countOfMessageIdentifier(int) countOfMessageIdentifier}
 * offers an easy way to iterate through each message to get the regular expressions used for matching.
 **/
   public String messageIdentity(int MessageIndex, int IdentityIndex) throws ChameleonException
   {
      return CHMengineMessageIdentifier(m_EngineHandle, MessageIndex, IdentityIndex);
   }
   
/**
* Get the name of the message at index <code>MessageIndex</code>.
* Usage of this method with {@link #countOfMessage() countOfMessage} offers
* an easy was to iterate through the messages.
**/
   public String messageName(int MessageIndex) throws ChameleonException
   {
      return CHMengineMessageName(m_EngineHandle, MessageIndex);
   }
/**
*  Get the Config object that allows one to set and get delimiter
*  characters.
**/
   public Config config() throws ChameleonException
   {
      return new Config(CHMengineConfigHandle(m_EngineHandle));
   }
/**
* Get a message in string format by generating it from the table structure <code>ATable</code>
* and the message at <code>MessageIndex</code>.
**/
   public String generate(Table ATable, int MessageIndex) throws ChameleonException
   {
      return CHMengineGenerateMessage(m_EngineHandle,
                                      MessageIndex,
                                      ATable.handle());
   }

   // package method
/**
* Create an empty message table with the specified message index.
**/
   public void createEmptyMessageTable(Table ATable, int MessageIndex) throws ChameleonException
   {
      CHMengineCreateEmptyMessageTable(m_EngineHandle, 
                                       MessageIndex, 
                                       ATable.handle());
   }
/**
* Get the log interface.
* @see #addLog(Log)
**/
   public Log log()
   {
      return m_Log;
   }

/**
* Add the specified log listener to receive log events from the Engine.
* This method is used by the Engine for logging output.
* Use of the python <code>log()</code> command in the VMD file will be output through this interface.
* @see #log()
**/
   public void addLog(Log ALog)
   {
      m_Log = ALog;
   }

/**
* Print the string <code>Line</code> to the log output.
**/
   public void println(String Line)
   {
      if (m_Log != null)
      {
         m_Log.println(Line);
      }
   }
/**
*  Convert a message from one format to another.
*  This method invokes Chameleon's translation logic to take a message of the
*  <code>InConfigIndex</code> flavor and translate it to the <code>OutConfigIndex</code>
*  flavor. The method returns the transformed data. 
*  @see <a href="http://www.interfaceware.com/manual/transform_message.html">http://www.interfaceware.com/manual/transform_message.html</a>
**/

   public String transformMessage
   (
      String InMessage,
      int    InConfigIndex,
      int    OutConfigIndex
   ) throws ChameleonException
   {
      return CHMengineTransformMessage(m_EngineHandle,
                                       InMessage,
                                       InConfigIndex,
                                       OutConfigIndex);
   }

/**
*  Get the number of configurations in the VMD file.  
*  This is useful for iterating through the named 
*  configurations.
**/

   public int countOfConfig() throws ChameleonException
   {
      return CHMengineGetCountOfConfig(m_EngineHandle);
   }

   /**
    *  Get the index of the current configuration.
    *  @see #setCurrentConfigIndex
    **/
   public int currentConfigIndex() throws ChameleonException
   {
      return CHMengineConfigIndex(m_EngineHandle);
   }

   /**
    * Set the index of the current configuration.
    * @see #currentConfigIndex()
    **/
   public void setCurrentConfigIndex(int NewIndex) throws ChameleonException
   {
      CHMengineSetConfigIndex(m_EngineHandle, NewIndex);
   }

   /**
    * Get the index of the incoming configuration used for graphical transformation.
    * The index is that of the input configuration last set by the user in the 
    * Transformation window in the VMD file.
    * <b>This is not used for scripted transformation.</b>
    **/
   public int inConfigIndex() throws ChameleonException
   {
      return CHMengineInConfigIndex(m_EngineHandle);
   }

   /**
    * Get the index of the outgoing configuration used for graphical transformation.
    * The index is that of the output configuration last set by the user in the
    * Transformation window in the VMD file.
    * <b>This is not used for scripted transformation.</b>
    **/
   public int outConfigIndex() throws ChameleonException
   {
      return CHMengineOutConfigIndex(m_EngineHandle);
   }

/**
*  Get the name of the configuration with the specified index.
**/
   
   public String configName(int ConfigIndex) throws ChameleonException
   {
      return CHMengineGetNameOfConfig(m_EngineHandle, ConfigIndex);
   }

/**
* Convert an error to a string and append the description. The
* resulting string is printed to the log output.
**/
   protected void errorEvent(Exception Error, String Description)
   {
      if (m_Log != null)
      {
         m_Log.println(Error.toString() + " : " + Description);        
      }
      else
      {
         System.out.println(Error.toString() + " : " + Description);
      }
   }

/**
*  Get the major version number of Chameleon.
**/
   public int majorVersion() throws ChameleonException
   {
      return CHMengineGetMajorVersion(m_EngineHandle);
   }

/**
*  Get the minor version number of Chameleon.
**/
   public int minorVersion() throws ChameleonException
   {
      return CHMengineGetMinorVersion(m_EngineHandle);
   }

/**
*  Get the build number of Chameleon.
**/
   public int buildNumber() throws ChameleonException
   {
      return CHMengineGetBuildNumber(m_EngineHandle);
   }

/**
*  Get the index of the message selected by the user to be the ACKnowledgment message.
*  This message is set in the Special Messages tab of the Options window in Chameleon.
*  @see <a href="http://www.interfaceware.com/manual/autoack.html">http://www.interfaceware.com/manual/autoack.html</a>
**/
   public int ackMessageIndex() throws ChameleonException
   {
      return CHMengineAckMessageIndex(m_EngineHandle);
   }

/**
*  Get the index of the message selected by the user to be the default message.
*  This message is used to identify messages which do not match the precise
*  identification criteria for other messages in the given VMD file.  
*  This message is set in the Special Messages tab of the
*  Options window in Chameleon.
*  @see <a href="http://www.interfaceware.com/manual/default_message.html">http://www.interfaceware.com/manual/default_message.html</a>
**/
   public int ignoreMessageIndex() throws ChameleonException
   {
      return CHMengineIgnoreMessageIndex(m_EngineHandle);
   }
/**
*  Generate a XML schema.
*  This schema is that used to translate to and from XML.
**/
   public void generateSchema( String Filename ) throws ChameleonException
   {
      CHMengineGenerateSchema( m_EngineHandle, Filename );
   }
/**
 *  Generate an ACKnowledgment message in an optimized manner.
 *  This function will strip off just the MSH segment of an HL7 message.
 *  It parses this segment, stores parameters in Python variables, then
 *  generates an ACK message.  The technique used is described as Auto 
 *  Acknowledgment in the manual. The function will have unpredictable effects if the message definition
 *  file is not set up correctly.
 *  @see <a href="http://www.interfaceware.com/manual/autoack.html">http://www.interfaceware.com/manual/autoack.html</a>
 **/
   public String generateAckFromMessage(String Message) throws ChameleonException
   {
      return CHMengineGenerateAckFromMessage(m_EngineHandle, Message);
   }

/**
 * Set a key value pair in the Python global namespace.  Certain reserved 
 * words will result in exceptions, i.e., "environment" as a key.  Calling 
 * this method will result in the variable <code>Key</code> in Python being assigned the
 * value <code>Value</code>.
 **/   
   public void setPythonKeyValuePair(String Key,String Value) throws ChameleonException
   {
      CHMengineSetPythonKeyValuePair(m_EngineHandle,Key,Value);
   }

   private native String CHMengineGenerateAckFromMessage(long EngineHandle, String Message) throws ChameleonException;

   // returns Engine handle
   private native long CHMengineCreate() throws ChameleonException;
   private native long CHMengineAddListener(long EngineHandle) throws ChameleonException;
   private native void CHMengineAddRef (long EngineHandle) throws ChameleonException;
   private native void CHMengineRelease(long EngineHandle, long EventHandle) throws ChameleonException;
   private native void CHMengineLoad   (long EngineHandle, String FileName) throws ChameleonException;

   // returns the MessageIndex
   private native int CHMengineParseMessage
   (
      long    EngineHandle, 
      String Flatwire,
      long    TableHandle
   ) throws ChameleonException;

   private native String CHMengineGenerateMessage
   (
      long    EngineHandle,
      int    MessageIndex,
      long    TableHandle
   ) throws ChameleonException;
   
   private native void CHMengineCreateEmptyMessageTable
   (
      long EngineHandle,
      int MessageIndex,
      long EmptyTable
   ) throws ChameleonException;
   
// These methods provide version information.

   private native int CHMengineGetMinorVersion
   (
      long EngineHandle
   ) throws ChameleonException;

   private native int CHMengineGetMajorVersion
   (
      long EngineHandle
   ) throws ChameleonException;

   private native int CHMengineGetBuildNumber
   (
      long EngineHandle
   ) throws ChameleonException;

   private native void CHMengineSetRejectBadSegmentGrammarFlag
   (
      long     EngineHandle,
      boolean Flag
   ) throws ChameleonException;

   private native int CHMengineCountOfMessage
   (
      long EngineHandle
   ) throws ChameleonException;

   private native String CHMengineMessageName
   (
      long EngineHandle,
      int MessageIndex
   ) throws ChameleonException;

   private native String CHMengineTransformMessage
   (
      long    EngineHandle,
      String InMessage,
      int    InConfigIndex,
      int    OutConfigIndex
   ) throws ChameleonException;

   private native int CHMengineGetCountOfConfig
   (
      long EngineHandle
   ) throws ChameleonException;

   private native String CHMengineGetNameOfConfig
   (
      long EngineHandle,
      int ConfigIndex
   ) throws ChameleonException;
   
   private native String CHMengineTranslateMessageToXml
   (
      long    EngineHandle,
      String FlatWire
   ) throws ChameleonException;

   private native String CHMengineTranslateXmlToMessage
   (
      long    EngineHandle,
      String Xml
   ) throws ChameleonException;

   
   private native String CHMengineMakeMessagePrintable(long EngineHandle, String Message) throws ChameleonException;
   private native int  CHMengineConfigIndex(long EngineHandle) throws ChameleonException;
   private native void CHMengineSetConfigIndex(long EngineHandle, int NewIndex) throws ChameleonException;
   private native int  CHMengineOutConfigIndex(long EngineHandle) throws ChameleonException;
   private native int  CHMengineInConfigIndex(long EngineHandle) throws ChameleonException;
   
   private native int CHMengineAckMessageIndex(long EngineHandle) throws ChameleonException;

   private native int CHMengineIgnoreMessageIndex(long EngineHandle) throws ChameleonException;

   private native long CHMengineConfigHandle(long EngineHandle) throws ChameleonException;
   private native String CHMengineMessageIdentifier(long EngineHandle, int MessageIndex, int IdentityIndex) throws ChameleonException;
   private native int CHMengineCountOfMessageIdentifier(long EngineHandle , int MessageIndex) throws ChameleonException;
   private native void CHMengineGenerateSchema(long EngineHandle, String Filename ) throws ChameleonException;
   private native void CHMengineSetPythonKeyValuePair(long EngineHandle,String Key,String Value) throws ChameleonException;

}
