package com.interfaceware.chameleon;

import java.util.*;


/**
 * <p>This class can be used by customers to automate the registration of Chameleon 
 * run-time instances. This class allows you to programmatically register Chameleon 
 * on a user's machine without any user input.</p>
 * 
 * <p>Beginning in version 4.5, this class allows you to register Chameleon without 
 * the machine being registered having a direct connection to the Internet. This is 
 * useful for companies that want to automate the registration code setup for 
 * Chameleon. To make it easy for companies to incorporate Chameleon into their 
 * existing licensing process, this class allows you to:</p>
 * 
 * <p> (1) Generate a unique Machine ID for the host machine.</p>
 * 
 * <p> (2) Request a non-expiring license for the Machine ID using the user name 
 * and password for the host members account. The automated registration process 
 * involves communication with an iNTERFACEWARE server via HTTP. This can be 
 * accomplished either from a centrally located computer, or from the actual host 
 * machine.</p>
 * 
 * <p> (3) Apply the registration code to the host machine. </p>
 * 
 * <p>This process makes it easy for companies to get the machine IDs centrally 
 * using their own licensing, meaning they can add the extra Chameleon information 
 * to their own license file.</p>
 * 
 * <p>Please see <a 
 * href="http://www.interfaceware.com/manual/api_registration.html">http://www.interfaceware.com/manual/api_registration.html</a>
 *  for details.</p>
 * 
 */
public class License
{
   private long LicenseHandle;
   
   public License() throws ChameleonException
   {
      LicenseHandle = CHMlicenseCreate();
      CHMlicenseAddRef(LicenseHandle);
   }
   
   public License(long Handle) throws ChameleonException
   {
      LicenseHandle = Handle;
      CHMlicenseAddRef(LicenseHandle);
   }
   
   public void release() throws ChameleonException
   {
      if (0 != LicenseHandle)
      {
         CHMlicenseRelease(LicenseHandle, 0);
         LicenseHandle = 0;
      }
   }
   
   protected void finalize() throws ChameleonException
   {
      release();
   }
   
   static
   {
      System.loadLibrary("chm_java");
   }
   
   
   /**
    * Get handle. This is part of the internal implementation of Chameleon and should 
    * not be called directly.
    */
   private long getHandle()
   {
      return LicenseHandle;
   }
   
   /**
    * <p>This is a boolean function, which returns true if the machine has a valid 
    * Chameleon license, and false otherwise. This method checks the license status of 
    * the target machine, but does not send a request to the iNTERFACEWARE server.</p>
    * 
    */
   public boolean getIsLicensed() throws ChameleonException
   {
      return CHMlicenseGetIsLicensed(LicenseHandle);
   }
   
   
   /**
    * <p>The IP address or hostname for the Web proxy to use. A Web proxy is required 
    * when the local machine does not have direct access to the Internet and needs to 
    * forward the registration requests through another server. A Web proxy server is 
    * a server that handles the HTTP requests of its clients by forwarding its 
    * requests to other servers (e.g. iNTERFACEWARE server). A client will connect to 
    * the Web proxy server to request a file. The proxy server then connects to the 
    * specified server and requests the file on behalf of the client. If your network 
    * requires your Internet connection to be set up via a Web proxy, you must specify 
    * the proxy your network uses to forward HTTP requests to other servers. By 
    * default, no proxy will be used. After setting a Web proxy, to unset it simply 
    * set this property to an empty string.</p>
    * 
    * <p>See also the {@link License#setProxy setProxy} method.</p>
    * 
    */
   public String getProxy() throws ChameleonException
   {
      return CHMlicenseGetProxy(LicenseHandle);
   }
   
   
   /**
    * <p>Set method for proxy.</p>
    * 
    * <p>See {@link License#getProxy getProxy}.</p>
    * 
    * @param   Value   Value to set proxy to.
    */
   public void setProxy(String Value) throws ChameleonException
   {
      CHMlicenseSetProxy(LicenseHandle, Value);
   }
   
   
   /**
    * <p>This method returns the unique machine ID of the machine to register.</p>
    * 
    */
   public String getMachineId() throws ChameleonException
   {
      return CHMlicenseGetMachineId(LicenseHandle);
   }
   
   
   /**
    * <p>If successful, this method will return the non-expiring runtime registration 
    * code for the unique machine ID from iNTERFACEWARE's server. This process 
    * involves an HTTP request to the iNTERFACEWARE server on port 80, therefore, 
    * there must be an available Internet connection. The HTTP request consists of 
    * your email address and password (the same one used to log into your member's 
    * account) and a machineId uniquely identifying the machine to register. The 
    * Description of the registration code will be used to identify it in order to 
    * determine the nature of the license. </p>
    * 
    * @param   MachineId   The Machine ID of the machine to register.
    * @param   EmailAddress   Email address used to login to iNTERFACEWARE's members 
    * site.
    * @param   Password   Password used to login to iNTERFACEWARE's members site.
    * @param   Description   A description of the registration code.
    */
   public String fetchRegistrationCode(String MachineId, String EmailAddress, String Password, String Description) throws ChameleonException
   {
      return CHMlicenseFetchRegistrationCode(LicenseHandle, MachineId, EmailAddress, Password, Description);
   }
   
   
   /**
    * <p>This is a method that is used to register the machine using the given 
    * registration code. This method checks if the registration key is valid and 
    * throws an exception if it is invalid; it does not send a request to the 
    * iNTERFACEWARE server. Also note that the method only supports non-expiring 
    * keys.</p>
    * 
    * @param   RegistrationCode   The registration code.
    */
   public void registerHost(String RegistrationCode) throws ChameleonException
   {
      CHMlicenseRegisterHost(LicenseHandle, RegistrationCode);
   }
   
   
   /**
    * <p>If successful, this method will return the expiring trial registration code 
    * for the unique machine ID from iNTERFACEWARE's server. This process involves an 
    * HTTP request to the iNTERFACEWARE server on port 80, therefore, there must be an 
    * available Internet connection. The HTTP request consists of your email address 
    * and password (the same one used to log into your member's account) and a 
    * machineId uniquely identifying the machine to register. The Description of the 
    * registration code will be used to identify it in order to determine the nature 
    * of the license. </p>
    * 
    * @param   ExpiryDate   Returns the expiry date of the registration code.
    * @param   MachineId   The Machine ID of the machine to register.
    * @param   EmailAddress   Email address used to login to iNTERFACEWARE's members 
    * site.
    * @param   Password   Password used to login to iNTERFACEWARE's members site.
    * @param   Description   A description of the registration code.
    */
   public String fetchTrialRegistrationCode(Date ExpiryDate, String MachineId, String EmailAddress, String Password, String Description) throws ChameleonException
   {
      return CHMlicenseFetchTrialRegistrationCode(LicenseHandle, ExpiryDate, MachineId, EmailAddress, Password, Description);
   }
   
   
   /**
    * <p>This is a method that is used to register the machine using the given 
    * expiring trial registration code and expiry date. This method checks if the 
    * registration key is valid and throws an exception if it is invalid; it does not 
    * send a request to the iNTERFACEWARE server.</p>
    * 
    * @param   RegistrationCode   The registration code.
    * @param   ExpiryDate   The expiry date of the registration code.
    */
   public void registerTrialHost(String RegistrationCode, Date ExpiryDate) throws ChameleonException
   {
      CHMlicenseRegisterTrialHost(LicenseHandle, RegistrationCode, ExpiryDate);
   }
   
   private native long CHMlicenseCreate() throws ChameleonException;
   private native void CHMlicenseAddRef(long LicenseHandle) throws ChameleonException;
   private native void CHMlicenseRelease(long LicenseHandle, long EventHandle) throws ChameleonException;
   
   private native boolean CHMlicenseGetIsLicensed(long LicenseHandle) throws ChameleonException;
   private native String CHMlicenseGetProxy(long LicenseHandle) throws ChameleonException;
   private native void CHMlicenseSetProxy(long LicenseHandle, String Value) throws ChameleonException;
   private native String CHMlicenseGetMachineId(long LicenseHandle) throws ChameleonException;
   private native String CHMlicenseFetchRegistrationCode(long LicenseHandle, String MachineId, String EmailAddress, String Password, String Description) throws ChameleonException;
   private native void CHMlicenseRegisterHost(long LicenseHandle, String RegistrationCode) throws ChameleonException;
   private native String CHMlicenseFetchTrialRegistrationCode(long LicenseHandle, Date ExpiryDate, String MachineId, String EmailAddress, String Password, String Description) throws ChameleonException;
   private native void CHMlicenseRegisterTrialHost(long LicenseHandle, String RegistrationCode, Date ExpiryDate) throws ChameleonException;
}