package com.interfaceware.chameleon;

import java.util.*;


/**
 * <p>This class can be used to implement a Lower Layer Protocol (LLP) TCP/IP 
 * client to connect to an LLP server.</p>
 * 
 * <p>See <a 
 * href="http://www.interfaceware.com/manual/network_components.html">http://www.interfaceware.com/manual/network_components.html</a>
 *  for more information.</p>
 * 
 */
public class LlpClient
{
   private long ClientHandle;
   private long EventHandle;
   private Vector ListenerVector;
   
   public LlpClient() throws ChameleonException
   {
      ClientHandle = CHMclientCreate();
      EventHandle = CHMclientAddListener(ClientHandle);
      ListenerVector = new Vector();
      CHMclientAddRef(ClientHandle);
   }
   
   public LlpClient(long Handle) throws ChameleonException
   {
      ClientHandle = Handle;
      CHMclientAddRef(ClientHandle);
   }
   
   public void release() throws ChameleonException
   {
      if (0 != ClientHandle)
      {
         CHMclientRelease(ClientHandle, EventHandle);
         ClientHandle = 0;
         EventHandle = 0;
         ListenerVector.clear();
      }
   }
   
   protected void finalize() throws ChameleonException
   {
      release();
   }
   
   static
   {
      System.loadLibrary("chm_java");
   }
   
   
   /**
    * Get handle. This is part of the internal implementation of Chameleon and should 
    * not be called directly.
    */
   private long getHandle()
   {
      return ClientHandle;
   }
   
   /**
    * <p>The characters that precede a message in a stream.</p>
    * 
    * <p>The default value for this property is 0x0B as defined by the the LLP 
    * protocol.</p>
    * 
    * <p>This property is used both for parsing out the incoming data stream and 
    * wrapping outgoing messages.</p>
    * 
    * <p>See also the {@link LlpClient#setHeaderBlock setHeaderBlock} method.</p>
    * 
    */
   public String getHeaderBlock() throws ChameleonException
   {
      return CHMclientGetHeaderBlock(ClientHandle);
   }
   
   
   /**
    * <p>Set method for headerBlock.</p>
    * 
    * <p>See {@link LlpClient#getHeaderBlock getHeaderBlock}.</p>
    * 
    * @param   Value   Value to set headerBlock to.
    */
   public void setHeaderBlock(String Value) throws ChameleonException
   {
      CHMclientSetHeaderBlock(ClientHandle, Value);
   }
   
   
   /**
    * <p>The characters that follow a message in a stream.</p>
    * 
    * <p>The default value is the character 0x1C followed by 0x0D as defined by the 
    * LLP protocol.</p>
    * 
    * <p>This property is used both for parsing out the incoming data stream and 
    * wrapping outgoing messages.</p>
    * 
    * <p>See also the {@link LlpClient#setTrailerBlock setTrailerBlock} method.</p>
    * 
    */
   public String getTrailerBlock() throws ChameleonException
   {
      return CHMclientGetTrailerBlock(ClientHandle);
   }
   
   
   /**
    * <p>Set method for trailerBlock.</p>
    * 
    * <p>See {@link LlpClient#getTrailerBlock getTrailerBlock}.</p>
    * 
    * @param   Value   Value to set trailerBlock to.
    */
   public void setTrailerBlock(String Value) throws ChameleonException
   {
      CHMclientSetTrailerBlock(ClientHandle, Value);
   }
   
   
   /**
    * <p>The status of the connection with the server.</p>
    * 
    * <p>Returns true if the client is currently connected with the server.</p>
    * 
    */
   public boolean getIsConnected() throws ChameleonException
   {
      return CHMclientGetIsConnected(ClientHandle);
   }
   
   
   /**
    * <p>The validity of the internal network socket handle</p>
    * 
    * <p>Returns true if the internal network socket handle is valid. The internal 
    * network socket handle is available via {@link LlpClient#getSocketHandle 
    * getSocketHandle}.</p>
    * 
    */
   public boolean getIsValidSocketHandle() throws ChameleonException
   {
      return CHMclientGetIsValidSocketHandle(ClientHandle);
   }
   
   
   /**
    * <p>The internal network socket handle.</p>
    * 
    * <p>Returns a handle to the internal network socket.</p>
    * 
    * <p>The internal network socket handle may be used to set advanced socket options 
    * via the platform socket API (WinSock2 on Windows and Berkeley Sockets on POSIX 
    * platforms).</p>
    * 
    */
   public int getSocketHandle() throws ChameleonException
   {
      return CHMclientGetSocketHandle(ClientHandle);
   }
   
   
   /**
    * <p> Send a quit message to the thread that the networking components are running 
    * on. </p>
    * 
    * <p> Under Windows, this is a WM_QUIT message. All the networking components must 
    * be created on a single thread with a running message loop. For more information, 
    * see the {@link LlpClient#runMessageLoop runMessageLoop} method. </p>
    * 
    */
   public void stopMessageLoop() throws ChameleonException
   {
      CHMclientStopMessageLoop(ClientHandle);
   }
   
   
   /**
    * <p>On Unix and Linux platforms, events are dispatched using a loop based on the 
    * ::select function. All POSIX applications require this function to be called in 
    * order to start receiving events.</p>
    * 
    * <p>On Windows platforms all events are dispatched using the Windows message 
    * loop.</p>
    * 
    * <p>Graphical Windows applications contain a running Windows message loop; 
    * therefore, this function will not need to be called. Console applications 
    * require this function to be called in order to start receiving events.</p>
    * 
    * <p>Throws: Function will throw an exception only if the operating system reports 
    * an error.
    * <br> Blocking: Function will block until {@link LlpClient#stopMessageLoop 
    * stopMessageLoop} is called, or (Windows only) a WM_QUIT message is posted to the 
    * message loop on the thread that the networking components are running on.</p>
    * 
    */
   static public void runMessageLoop() throws ChameleonException
   {
      CHMclientRunMessageLoop();
   }
   
   
   /**
    * <p>Make message printable by changing plain carriage return characters "\r" to 
    * carriage return and newline characters "\r\n".</p>
    * 
    * <p>HL7 messages use the carriage return character "\r" to separate the segments. 
    * This makes it difficult to print them out in console windows because the cursor 
    * overwrites each previous segment instead of moving on to a newline for each 
    * segment. This method is intended to be a simple utility method to make it easier 
    * to convert HL7 messages into a printable format. This function also allows the 
    * output string to be truncated.</p>
    * 
    * @param   InputString   Input message string.
    * @param   NewLine   Line feed string.
    * @param   MaximumLength   Maximum length of the message to print out. Characters 
    * after this length will be truncated. If you want to output the whole message, 
    * set the length to -1.
    * @return   Printable string, with "\r" characters changed to "\r\n"
    */
   public String makeMessagePrintable(String InputString, String NewLine, int MaximumLength) throws ChameleonException
   {
      return CHMclientMakePrintable(ClientHandle, InputString, NewLine, MaximumLength);
   }
   
   
   /**
    * <p>Connect to the given host and port number.</p>
    * 
    * <p>Function requires a running message loop (see {@link LlpClient#runMessageLoop 
    * runMessageLoop}).</p>
    * 
    * <p>Blocking: Function does not block. 
    * <br>Events: If the connection succeeds, a {@link LlpClientListener#connected 
    * connected} event results. If the connection fails, an {@link 
    * LlpClientListener#errorReceived errorReceived} event results, containing the 
    * error message indicating why the connect attempt has failed.</p>
    * 
    * @param   Host   Host to connect to.
    * @param   Port   Port to connect to.
    */
   public void connect(String Host, int Port) throws ChameleonException
   {
      CHMclientConnect(ClientHandle, Host, Port);
   }
   
   
   /**
    * <p>Attempt to gracefully disconnect from the server.</p>
    * 
    * <p>A graceful disconnect implies a shutdown to the write end of the connection, 
    * and reading all data until the peer issues a close to its write end.</p>
    * 
    * <p>Note that the peer may not actually close the write end of its connection 
    * (although this is rare), which results in the connection never receiving the 
    * close event to signify that the connection is closed. In this situation, it is 
    * recommended that {@link LlpClient#hardDisconnect hardDisconnect} be called to 
    * close the connection.</p>
    * 
    * <p>Function requires a running message loop (see {@link LlpClient#runMessageLoop 
    * runMessageLoop}).</p>
    * 
    * <p>Blocking: Function does not block. 
    * <br>Events: If the close succeeds (the peer closes the connection), a {@link 
    * LlpClientListener#closed closed} event results. If an error occurs during the 
    * close operation, an {@link LlpClientListener#errorReceived errorReceived} event 
    * results, containing the error message indicating why the close attempt has 
    * failed.</p>
    * 
    */
   public void disconnect() throws ChameleonException
   {
      CHMclientClose(ClientHandle);
   }
   
   
   /**
    * <p>Ungracefully disconnect without doing TCP/IP handshaking with the server.</p>
    * 
    * <p>If there is data pending on the network buffer to be sent, the peer will 
    * receive a reset on the connection if it attempts to read or write to the 
    * connection.</p>
    * 
    * <p>Blocking: Function blocks; upon returning, the connection is closed. 
    * <br>Events: None.</p>
    * 
    */
   public void hardDisconnect() throws ChameleonException
   {
      CHMclientHardClose(ClientHandle);
   }
   
   
   /**
    * <p>Send a message to the server, prefixed and postfixed by the current {@link 
    * LlpClient#getHeaderBlock getHeaderBlock} and {@link LlpClient#getTrailerBlock 
    * getTrailerBlock} characters.</p>
    * 
    * <p>The data is not guaranteed to be received after this function is called. All 
    * data is buffered internally; completion of this call does not signify reception 
    * of data.</p>
    * 
    * <p>Function requires a running message loop (see {@link LlpClient#runMessageLoop 
    * runMessageLoop}).</p>
    * 
    * <p>Blocking: Function does not block. 
    * <br>Events: If an error occurs during the send operation an {@link 
    * LlpClientListener#errorReceived errorReceived} event results, containing the 
    * error message indicating why the send attempt has failed.</p>
    * 
    * @param   Message   Message to send to the server.
    */
   public void transmitMessage(String Message) throws ChameleonException
   {
      CHMclientSendMessage(ClientHandle, Message);
   }
   
   /**
    * <p>Fired when the connection is made with the server.</p>
    * 
    */
   private void connected()
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpClientListener)ListenerVector.elementAt(ListenerIndex)).connected();
      }
   }
   
   /**
    * <p>Fired when the connection is closed.</p>
    * 
    */
   private void closed()
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpClientListener)ListenerVector.elementAt(ListenerIndex)).closed();
      }
   }
   
   /**
    * <p>Fired when a message is received.</p>
    * 
    * <p>Does not include the {@link LlpClient#getHeaderBlock getHeaderBlock} and 
    * {@link LlpClient#getTrailerBlock getTrailerBlock} characters.</p>
    * 
    */
   private void messageReceived(String Message)
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpClientListener)ListenerVector.elementAt(ListenerIndex)).messageReceived(Message);
      }
   }
   
   /**
    * <p>Fired when data has been received that is not within the header and trailer 
    * block.</p>
    * 
    * <p>This is for dealing with an incorrect LLP implementation. The data is 
    * provided to the user's application through this event. This allows it to be 
    * logged since this is likely to be a bad error.</p>
    * 
    */
   private void dataIgnored(String Data)
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpClientListener)ListenerVector.elementAt(ListenerIndex)).dataIgnored(Data);
      }
   }
   
   /**
    * <p>Fired when we have an error with the TCP/IP connection.</p>
    * 
    * <p>The context of when this is received is often determined by a previous 
    * operation. The connection will always be closed once this event is received.</p>
    * 
    */
   private void errorReceived(String Description, int Code)
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpClientListener)ListenerVector.elementAt(ListenerIndex)).errorReceived(Description, Code);
      }
   }
   
   /**
    * <p>Adds the specified listener so that it receives events from this LlpClient 
    * object.</p>
    * 
    * @see  #removeListener(LlpClientListener)  removeListener
    */
   public void addListener(LlpClientListener Listener) throws ChameleonException
   {
      ListenerVector.add(Listener);
   }
   
   
   /**
    * <p>Removes the specified listener so that it no longer receives events from this 
    * LlpClient object.</p>
    * 
    * @see  #addListener(LlpClientListener)  addListener
    */
   public void removeListener(LlpClientListener Listener) throws ChameleonException
   {
      ListenerVector.remove(Listener);
   }
   private native long CHMclientCreate() throws ChameleonException;
   private native long CHMclientAddListener(long ClientHandle) throws ChameleonException;
   private native void CHMclientAddRef(long ClientHandle) throws ChameleonException;
   private native void CHMclientRelease(long ClientHandle, long EventHandle) throws ChameleonException;
   
   private native String CHMclientGetHeaderBlock(long ClientHandle) throws ChameleonException;
   private native void CHMclientSetHeaderBlock(long ClientHandle, String Value) throws ChameleonException;
   private native String CHMclientGetTrailerBlock(long ClientHandle) throws ChameleonException;
   private native void CHMclientSetTrailerBlock(long ClientHandle, String Value) throws ChameleonException;
   private native boolean CHMclientGetIsConnected(long ClientHandle) throws ChameleonException;
   private native boolean CHMclientGetIsValidSocketHandle(long ClientHandle) throws ChameleonException;
   private native int CHMclientGetSocketHandle(long ClientHandle) throws ChameleonException;
   private native void CHMclientStopMessageLoop(long ClientHandle) throws ChameleonException;static 
   private native void CHMclientRunMessageLoop() throws ChameleonException;
   private native String CHMclientMakePrintable(long ClientHandle, String InputString, String NewLine, int MaximumLength) throws ChameleonException;
   private native void CHMclientConnect(long ClientHandle, String Host, int Port) throws ChameleonException;
   private native void CHMclientClose(long ClientHandle) throws ChameleonException;
   private native void CHMclientHardClose(long ClientHandle) throws ChameleonException;
   private native void CHMclientSendMessage(long ClientHandle, String Message) throws ChameleonException;
}