package com.interfaceware.chameleon;



/**
 * <p>The listener interface for receiving {@link LlpClient LlpClient} events. A 
 * class that is interested in processing a LlpClient event must implement this 
 * interface (and all the methods it contains). The listener object created from 
 * that class is then registered with a LlpClient object using the LlpClient's 
 * {@link LlpClient#addListener addListener} method. When the LlpClient's state 
 * changes, the relevant method in the listener object is invoked. If you wish the 
 * LlpClient object to stop receiving notifications, use the LlpClient's {@link 
 * LlpClient#removeListener removeListener} method.</p>
 * 
 * @see  LlpClient
 */
public interface LlpClientListener
{
   
   /**
    * <p>Fired when the connection is made with the server.</p>
    * 
    */
   public void connected();
   
   /**
    * <p>Fired when the connection is closed.</p>
    * 
    */
   public void closed();
   
   /**
    * <p>Fired when a message is received.</p>
    * 
    * <p>Does not include the {@link LlpClient#getHeaderBlock getHeaderBlock} and 
    * {@link LlpClient#getTrailerBlock getTrailerBlock} characters.</p>
    * 
    * @param   Message   The message received.
    */
   public void messageReceived(String Message);
   
   /**
    * <p>Fired when data has been received that is not within the header and trailer 
    * block.</p>
    * 
    * <p>This is for dealing with an incorrect LLP implementation. The data is 
    * provided to the user's application through this event. This allows it to be 
    * logged since this is likely to be a bad error.</p>
    * 
    * @param   Data   The ignored data.
    */
   public void dataIgnored(String Data);
   
   /**
    * <p>Fired when we have an error with the TCP/IP connection.</p>
    * 
    * <p>The context of when this is received is often determined by a previous 
    * operation. The connection will always be closed once this event is received.</p>
    * 
    * @param   Description   Description of the error.
    * @param   Code   Error code.
    */
   public void errorReceived(String Description, int Code);
   
}