package com.interfaceware.chameleon;

import java.util.*;


/**
 * <p>This object is passed as an argument for many of the events of the {@link 
 * LlpServer LlpServer} object.</p>
 * 
 * <p>This class provides methods to interact with each discrete client 
 * connection.</p>
 * 
 * <p>See <a 
 * href="http://www.interfaceware.com/manual/network_components.html">http://www.interfaceware.com/manual/network_components.html</a>
 *  for more information.</p>
 * 
 */
public class LlpConnection
{
   private long ConnectionHandle;
   
   public LlpConnection() throws ChameleonException
   {
      ConnectionHandle = CHMconnectionCreate();
      CHMconnectionAddRef(ConnectionHandle);
   }
   
   public LlpConnection(long Handle) throws ChameleonException
   {
      ConnectionHandle = Handle;
      CHMconnectionAddRef(ConnectionHandle);
   }
   
   public void release() throws ChameleonException
   {
      if (0 != ConnectionHandle)
      {
         CHMconnectionRelease(ConnectionHandle, 0);
         ConnectionHandle = 0;
      }
   }
   
   protected void finalize() throws ChameleonException
   {
      release();
   }
   
   static
   {
      System.loadLibrary("chm_java");
   }
   
   
   /**
    * Get handle. This is part of the internal implementation of Chameleon and should 
    * not be called directly.
    */
   private long getHandle()
   {
      return ConnectionHandle;
   }
   
   /**
    * <p> This property will resolve the remoteIpAddress to the host name if possible 
    * using the operating system. Sometimes no host name resolution will be possible 
    * at which point this function will return 'unknown'. </p>
    * 
    * <p>For the first time this method is called, it blocks until the name is 
    * resolved. This means that if you make the call in the context of your message 
    * loop your application will appear to freeze for a few seconds. </p>
    * 
    * <p>Subsequent calls will return immediately with a cached value. </p>
    * 
    * <p> For more advanced functionality you may wish to use other facilities to 
    * resolve the host name outside of the functionality offered here. </p>
    * 
    */
   public String getRemoteHostName() throws ChameleonException
   {
      return CHMconnectionGetRemoteHostName(ConnectionHandle);
   }
   
   
   /**
    * <p>A unique ID for all connection objects.</p>
    * 
    * <p>This ID is generated sequentially (i.e. 0, 1, 2, 3, etc.) for all connections 
    * received from a LlpServer, beginning at 0 when the application starts. The ID is 
    * globally unique and will allow users to identify connections from the same 
    * client, or connections from different clients.</p>
    * 
    */
   public int getId() throws ChameleonException
   {
      return CHMconnectionGetId(ConnectionHandle);
   }
   
   
   /**
    * <p>The IP address of the remote host in host byte order.</p>
    * 
    * <p>Consider using {@link LlpConnection#getRemoteIpAddressAsString 
    * getRemoteIpAddressAsString} instead for a more usable format.</p>
    * 
    */
   public int getRemoteIpAddress() throws ChameleonException
   {
      return CHMconnectionGetRemoteIpAddress(ConnectionHandle);
   }
   
   
   /**
    * <p>The IP address of the remote host as a string.</p>
    * 
    * <p>See also {@link LlpConnection#getRemoteIpAddress getRemoteIpAddress}.</p>
    * 
    */
   public String getRemoteIpAddressAsString() throws ChameleonException
   {
      return CHMconnectionGetRemoteIpAddressAsString(ConnectionHandle);
   }
   
   
   /**
    * <p>The validity of the internal network socket handle</p>
    * 
    * <p>Returns true if the internal network socket handle is valid. The internal 
    * network socket handle is available via {@link LlpConnection#getSocketHandle 
    * getSocketHandle}.</p>
    * 
    */
   public boolean getIsValidSocketHandle() throws ChameleonException
   {
      return CHMconnectionGetIsValidSocketHandle(ConnectionHandle);
   }
   
   
   /**
    * <p>The internal network socket handle.</p>
    * 
    * <p>Returns a handle to the internal network socket.</p>
    * 
    * <p>The internal network socket handle may be used to set advanced socket options 
    * via the platform socket API (WinSock2 on Windows and Berkeley Sockets on POSIX 
    * platforms).</p>
    * 
    */
   public int getSocketHandle() throws ChameleonException
   {
      return CHMconnectionGetSocketHandle(ConnectionHandle);
   }
   
   
   /**
    * <p>Send a message to the client, prefixed and postfixed by the current {@link 
    * LlpServer#getHeaderBlock getHeaderBlock} and {@link LlpServer#getTrailerBlock 
    * getTrailerBlock} characters.</p>
    * 
    * <p>The data is not guaranteed to be received after this function is called. All 
    * data is buffered internally; completion of this call does not signify reception 
    * of data.</p>
    * 
    * <p>Function requires a running message loop (see {@link LlpServer#runMessageLoop 
    * runMessageLoop}).</p>
    * 
    * <p>Blocking: Function does not block. 
    * <br>Events: If an error occurs during the send operation, an {@link 
    * LlpServerListener#clientErrorReceived clientErrorReceived} event results, 
    * containing the error message indicating why the send attempt has failed.</p>
    * 
    * @param   Message   HL7 message to send to a client connection in plain string 
    * format.
    */
   public void transmitMessage(String Message) throws ChameleonException
   {
      CHMconnectionSendMessage(ConnectionHandle, Message);
   }
   
   
   /**
    * <p>Attempt to gracefully disconnect from the server.</p>
    * 
    * <p>A graceful disconnect implies a shutdown to the write end of the connection, 
    * and reading all data until the peer issues a close to its write end.</p>
    * 
    * <p>Note that the peer may not actually close the write end of its connection 
    * (although this is rare), which results in the connection never receiving the 
    * close event to signify that the connection is closed. In this situation, it is 
    * recommended that {@link LlpConnection#hardDisconnect hardDisconnect} be called 
    * to close the connection.</p>
    * 
    * <p>Function requires a running message loop (see {@link LlpServer#runMessageLoop 
    * runMessageLoop}).</p>
    * 
    * <p>Blocking: Function does not block. 
    * <br>Events: If the close succeeds (the peer closes the connection), a {@link 
    * LlpServerListener#clientDisconnected clientDisconnected} event results. If an 
    * error occurs during the close operation, an {@link 
    * LlpServerListener#clientErrorReceived clientErrorReceived} event results, 
    * containing the error message indicating why the close attempt failed.</p>
    * 
    */
   public void disconnect() throws ChameleonException
   {
      CHMconnectionClose(ConnectionHandle);
   }
   
   
   /**
    * <p>Ungracefully disconnect without doing TCP/IP handshaking with the server.</p>
    * 
    * <p>If there is data pending on the network buffer to be sent, the peer will 
    * receive a reset on the connection if it attempts to read or write to the 
    * connection.</p>
    * 
    * <p>Blocking: Function blocks; upon returning, the connection is closed. 
    * <br>Events: None.</p>
    * 
    */
   public void hardDisconnect() throws ChameleonException
   {
      CHMconnectionHardClose(ConnectionHandle);
   }
   
   private native long CHMconnectionCreate() throws ChameleonException;
   private native void CHMconnectionAddRef(long ConnectionHandle) throws ChameleonException;
   private native void CHMconnectionRelease(long ConnectionHandle, long EventHandle) throws ChameleonException;
   
   private native String CHMconnectionGetRemoteHostName(long ConnectionHandle) throws ChameleonException;
   private native int CHMconnectionGetId(long ConnectionHandle) throws ChameleonException;
   private native int CHMconnectionGetRemoteIpAddress(long ConnectionHandle) throws ChameleonException;
   private native String CHMconnectionGetRemoteIpAddressAsString(long ConnectionHandle) throws ChameleonException;
   private native boolean CHMconnectionGetIsValidSocketHandle(long ConnectionHandle) throws ChameleonException;
   private native int CHMconnectionGetSocketHandle(long ConnectionHandle) throws ChameleonException;
   private native void CHMconnectionSendMessage(long ConnectionHandle, String Message) throws ChameleonException;
   private native void CHMconnectionClose(long ConnectionHandle) throws ChameleonException;
   private native void CHMconnectionHardClose(long ConnectionHandle) throws ChameleonException;
}