package com.interfaceware.chameleon;

import java.util.*;


/**
 * <p>This object implements a Lower Layer Protocol (LLP) TCP/IP server that 
 * listens for connections from clients.</p>
 * 
 * <p>Most of the network events that are raised provide a {@link LlpConnection 
 * LlpConnection} parameter, which gives methods to interact with the client 
 * connection that the event is associated with.</p>
 * 
 * <p>See <a 
 * href="http://www.interfaceware.com/manual/network_components.html">http://www.interfaceware.com/manual/network_components.html</a>
 *  for more information.</p>
 * 
 */
public class LlpServer
{
   private long ListenerHandle;
   private long EventHandle;
   private Vector ListenerVector;
   
   public LlpServer() throws ChameleonException
   {
      ListenerHandle = CHMlistenerCreate();
      EventHandle = CHMlistenerAddListener(ListenerHandle);
      ListenerVector = new Vector();
      CHMlistenerAddRef(ListenerHandle);
   }
   
   public LlpServer(long Handle) throws ChameleonException
   {
      ListenerHandle = Handle;
      CHMlistenerAddRef(ListenerHandle);
   }
   
   public void release() throws ChameleonException
   {
      if (0 != ListenerHandle)
      {
         CHMlistenerRelease(ListenerHandle, EventHandle);
         ListenerHandle = 0;
         EventHandle = 0;
         ListenerVector.clear();
      }
   }
   
   protected void finalize() throws ChameleonException
   {
      release();
   }
   
   static
   {
      System.loadLibrary("chm_java");
   }
   
   
   /**
    * Get handle. This is part of the internal implementation of Chameleon and should 
    * not be called directly.
    */
   private long getHandle()
   {
      return ListenerHandle;
   }
   
   /**
    * <p>The port on which we are listening, set by the {@link LlpServer#startListen 
    * startListen} method.</p>
    * 
    */
   public int getPort() throws ChameleonException
   {
      return CHMlistenerGetPort(ListenerHandle);
   }
   
   
   /**
    * <p>The local interface on which we are listening, used by the next call to the 
    * {@link LlpServer#startListen startListen} method.</p>
    * 
    * <p>The default is an empty string, which means the listener will listen on all 
    * local interfaces.</p>
    * 
    * <p>It must be in the Internet standard "." (dotted) notation (e.g. 
    * 127.0.0.1).</p>
    * 
    * <p>See also the {@link LlpServer#setHost setHost} method.</p>
    * 
    */
   public String getHost() throws ChameleonException
   {
      return CHMlistenerGetHost(ListenerHandle);
   }
   
   
   /**
    * <p>Set method for host.</p>
    * 
    * <p>See {@link LlpServer#getHost getHost}.</p>
    * 
    * @param   Value   Value to set host to.
    */
   public void setHost(String Value) throws ChameleonException
   {
      CHMlistenerSetHost(ListenerHandle, Value);
   }
   
   
   /**
    * <p>The characters that precede a message in a stream.</p>
    * 
    * <p>The default value for this property is 0x0B as defined by the the LLP 
    * protocol.</p>
    * 
    * <p>This property is used both for parsing out the incoming data streams and 
    * wrapping outgoing messages.</p>
    * 
    * <p>See also the {@link LlpServer#setHeaderBlock setHeaderBlock} method.</p>
    * 
    */
   public String getHeaderBlock() throws ChameleonException
   {
      return CHMlistenerGetHeaderBlock(ListenerHandle);
   }
   
   
   /**
    * <p>Set method for headerBlock.</p>
    * 
    * <p>See {@link LlpServer#getHeaderBlock getHeaderBlock}.</p>
    * 
    * @param   Value   Value to set headerBlock to.
    */
   public void setHeaderBlock(String Value) throws ChameleonException
   {
      CHMlistenerSetHeaderBlock(ListenerHandle, Value);
   }
   
   
   /**
    * <p>The characters that follow a message in a stream.</p>
    * 
    * <p>The default value is the character 0x1C followed by 0x0D as defined by the 
    * LLP protocol.</p>
    * 
    * <p>This property is used both for parsing out the incoming data streams and 
    * wrapping outgoing messages.</p>
    * 
    * <p>See also the {@link LlpServer#setTrailerBlock setTrailerBlock} method.</p>
    * 
    */
   public String getTrailerBlock() throws ChameleonException
   {
      return CHMlistenerGetTrailerBlock(ListenerHandle);
   }
   
   
   /**
    * <p>Set method for trailerBlock.</p>
    * 
    * <p>See {@link LlpServer#getTrailerBlock getTrailerBlock}.</p>
    * 
    * @param   Value   Value to set trailerBlock to.
    */
   public void setTrailerBlock(String Value) throws ChameleonException
   {
      CHMlistenerSetTrailerBlock(ListenerHandle, Value);
   }
   
   
   /**
    * <p>The status of the listening state.</p>
    * 
    * <p>Returns true if the server is currently listening for connections. </p>
    * 
    * <p>Blocking: Function blocks until the listening state is determined. 
    * <br>Events: None.</p>
    * 
    */
   public boolean getIsListening() throws ChameleonException
   {
      return CHMlistenerGetIsListening(ListenerHandle);
   }
   
   
   /**
    * <p>The validity of the internal network socket handle</p>
    * 
    * <p>Returns true if the internal network socket handle is valid. The internal 
    * network socket handle is available via {@link LlpServer#getSocketHandle 
    * getSocketHandle}.</p>
    * 
    */
   public boolean getIsValidSocketHandle() throws ChameleonException
   {
      return CHMlistenerGetIsValidSocketHandle(ListenerHandle);
   }
   
   
   /**
    * <p>The internal network socket handle.</p>
    * 
    * <p>Returns a handle to the internal network socket.</p>
    * 
    * <p>The internal network socket handle may be used to set advanced socket options 
    * via the platform socket API (WinSock2 on Windows and Berkeley Sockets on POSIX 
    * platforms).</p>
    * 
    */
   public int getSocketHandle() throws ChameleonException
   {
      return CHMlistenerGetSocketHandle(ListenerHandle);
   }
   
   
   /**
    * <p>Representation of the collection of remote Clients that are connected to this 
    * server.</p>
    * 
    * <p>This list changes in size as connections are received or closed, or if an 
    * error occurs on the connection (which implies a close).</p>
    * 
    * <p>Users cannot change the size or append to the contents of this collection 
    * since it is managed internally.</p>
    * 
    * <p>References to {@link LlpConnection LlpConnection} objects within this 
    * collection should not be maintained. Instead, always access {@link LlpConnection 
    * LlpConnection} objects through this collection interface, or the event to which 
    * {@link LlpConnection LlpConnection} objects are passed as parameters.</p>
    * 
    * @param   ClientIndex   The index of the Client to return.
    */
   public LlpConnection getClient(int ClientIndex) throws ChameleonException
   {
      return CHMlistenerGetConnection(ListenerHandle, ClientIndex);
   }
   
   
   /**
    * <p>Returns the total number of Clients.</p>
    * 
    * <p>See also the {@link LlpServer#getClient getClient} property.</p>
    * 
    */
   public int countConnections() throws ChameleonException
   {
      return CHMlistenerCountOfConnection(ListenerHandle);
   }
   
   
   /**
    * <p> This will send a quit message to the thread that the networking components 
    * are running on. </p>
    * 
    * <p> Under Windows, this is a WM_QUIT message. All the networking components must 
    * be created on a single thread with a running message loop. For more information, 
    * see the {@link LlpServer#runMessageLoop runMessageLoop} method. </p>
    * 
    */
   public void stopMessageLoop() throws ChameleonException
   {
      CHMlistenerStopMessageLoop(ListenerHandle);
   }
   
   
   /**
    * <p>On Unix and Linux platforms events are dispatched using a loop based on the 
    * ::select function. All POSIX applications require this function to be called in 
    * order to start receiving events.</p>
    * 
    * <p>On Windows platforms all events are dispatched using the Windows message 
    * loop.</p>
    * 
    * <p>Graphical Windows applications contain a running Windows message loop; 
    * therefore, this function will not need to be called. Console applications 
    * require this function to be called in order to start receiving events.</p>
    * 
    * <p>Throws: Function will throw an exception only if the operating system reports 
    * an error.
    * <br> Blocking: Function will block until {@link LlpServer#stopMessageLoop 
    * stopMessageLoop} is called, or (Windows only) a WM_QUIT message is posted to the 
    * message loop on the thread that the networking components are running on.</p>
    * 
    */
   static public void runMessageLoop() throws ChameleonException
   {
      CHMlistenerRunMessageLoop();
   }
   
   
   /**
    * <p>Attempt to listen on the supplied port for incoming client connections.</p>
    * 
    * <p>This function requires a running message loop (see {@link 
    * LlpServer#runMessageLoop runMessageLoop}) in order to receive connections.</p>
    * 
    * <p>Blocking: Function blocks until a port can be successfully listened on or an 
    * error occurs. 
    * <br>Throws: Function will throw an exception if the port is in use, or the 
    * operating system reports any other error. 
    * <br>Events: None.</p>
    * 
    * <p>By default, the listener will listen on all local interfaces. To make it 
    * listen on a specific local interface use the {@link LlpServer#getHost getHost} 
    * property to set the IP address of the interface to use.</p>
    * 
    * @param   Port   Port to listen on.
    */
   public void startListen(int Port) throws ChameleonException
   {
      CHMlistenerStartListen(ListenerHandle, Port);
   }
   
   
   /**
    * <p>Make message printable by changing plain carriage return characters "\r" to 
    * carriage return and newline characters "\r\n".</p>
    * 
    * <p>HL7 messages use the carriage return character "\r" to separate the segments. 
    * This makes it difficult to print them out in console windows because the cursor 
    * overwrites each previous segment instead of moving on to a newline for each 
    * segment. This method is intended to be a simple utility method to make it easier 
    * to convert HL7 messages into a printable format. This function also allows the 
    * output string to be truncated.</p>
    * 
    * @param   InputString   Input message string.
    * @param   NewLine   Line feed string.
    * @param   MaximumLength   Maximum length of the message to print out. Characters 
    * after this length will be truncated. If you want to output the whole message, 
    * set the length to -1.
    * @return   Printable string, with "\r" characters changed to "\r\n"
    */
   public String makeMessagePrintable(String InputString, String NewLine, int MaximumLength) throws ChameleonException
   {
      return CHMlistenerMakePrintable(ListenerHandle, InputString, NewLine, MaximumLength);
   }
   
   
   /**
    * <p>Stop the component from listening for new connections.</p>
    * 
    * <p>Note that any existing connections that have already been established are NOT 
    * closed. In order to properly disconnect existing connections as well as stop 
    * listening, use {@link LlpServer#disconnectAllClients disconnectAllClients}.</p>
    * 
    * <p>Blocking: Function blocks until the stop listen succeeds. 
    * <br>Throws: Function does not throw any exceptions. 
    * <br>Events: None.</p>
    * 
    */
   public void stopListen() throws ChameleonException
   {
      CHMlistenerStopListen(ListenerHandle);
   }
   
   
   /**
    * <p>Stop listening on the port for new connections, and initiate a {@link 
    * LlpConnection#disconnect disconnect} on all open connections.</p>
    * 
    * <p>This function requires a running message loop (see {@link 
    * LlpServer#runMessageLoop runMessageLoop}).</p>
    * 
    * <p>Blocking: Function blocks until {@link LlpConnection#disconnect disconnect} 
    * is called on all open connections. This implies that the connections are not 
    * necessarily closed when this function returns. 
    * <br>Events: None.</p>
    * 
    */
   public void disconnectAllClients() throws ChameleonException
   {
      CHMlistenerCloseAll(ListenerHandle);
   }
   
   /**
    * <p>Fired when an issue occurs with the server socket itself rather than a 
    * specific client.</p>
    * 
    */
   private void errorReceived(String ErrorMessage, int Code)
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpServerListener)ListenerVector.elementAt(ListenerIndex)).errorReceived(ErrorMessage, Code);
      }
   }
   
   /**
    * <p>Fired when a connection is made to the server by a new client.</p>
    * 
    * <p>This event is useful for logging.</p>
    * 
    */
   private void clientConnected(LlpConnection Client)
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpServerListener)ListenerVector.elementAt(ListenerIndex)).clientConnected(Client);
      }
   }
   
   /**
    * <p>Fired when a client disconnects from the server, or a previous {@link 
    * LlpConnection#disconnect disconnect} call on a Connection succeeds.</p>
    * 
    * <p>This event is useful for logging.</p>
    * 
    */
   private void clientDisconnected(LlpConnection Client)
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpServerListener)ListenerVector.elementAt(ListenerIndex)).clientDisconnected(Client);
      }
   }
   
   /**
    * <p>Fired when a networking error occurs with a specific client connection.</p>
    * 
    * <p>The context on when this is received is often determined by a previous 
    * operation, although that is not necessary. The connection will always be closed 
    * once this event is received so there is no need to close the connection within 
    * the application code.</p>
    * 
    */
   private void clientErrorReceived(LlpConnection Client, String ErrorMessage, int ErrorCode)
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpServerListener)ListenerVector.elementAt(ListenerIndex)).clientErrorReceived(Client, ErrorMessage, ErrorCode);
      }
   }
   
   /**
    * <p>Fired each time we receive a message from a connected HL7 client.</p>
    * 
    * <p>Typically, the message should be processed and only then should an HL7 
    * ACKnowledgement message be sent.</p>
    * 
    * <p>Received without any Trailer or Header characters.</p>
    * 
    */
   private void messageReceived(LlpConnection Client, String Message)
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpServerListener)ListenerVector.elementAt(ListenerIndex)).messageReceived(Client, Message);
      }
   }
   
   /**
    * <p>Fired when data has been received that is not within the header and trailer 
    * block envelope.</p>
    * 
    * <p>This is for dealing with an incorrect LLP implementation. The data is 
    * provided to the user's application through this event. This allows it to be 
    * logged since this is likely to be a very significant error with the 
    * counterparty's LLP implementation.</p>
    * 
    */
   private void dataIgnored(LlpConnection Client, String Data)
   {
      for (int ListenerIndex = 0; ListenerIndex < ListenerVector.size(); ListenerIndex ++)
      {
         ((LlpServerListener)ListenerVector.elementAt(ListenerIndex)).dataIgnored(Client, Data);
      }
   }
   
   /**
    * <p>Adds the specified listener so that it receives events from this LlpServer 
    * object.</p>
    * 
    * @see  #removeListener(LlpServerListener)  removeListener
    */
   public void addListener(LlpServerListener Listener) throws ChameleonException
   {
      ListenerVector.add(Listener);
   }
   
   
   /**
    * <p>Removes the specified listener so that it no longer receives events from this 
    * LlpServer object.</p>
    * 
    * @see  #addListener(LlpServerListener)  addListener
    */
   public void removeListener(LlpServerListener Listener) throws ChameleonException
   {
      ListenerVector.remove(Listener);
   }
   private native long CHMlistenerCreate() throws ChameleonException;
   private native long CHMlistenerAddListener(long ListenerHandle) throws ChameleonException;
   private native void CHMlistenerAddRef(long ListenerHandle) throws ChameleonException;
   private native void CHMlistenerRelease(long ListenerHandle, long EventHandle) throws ChameleonException;
   
   private native int CHMlistenerGetPort(long ListenerHandle) throws ChameleonException;
   private native String CHMlistenerGetHost(long ListenerHandle) throws ChameleonException;
   private native void CHMlistenerSetHost(long ListenerHandle, String Value) throws ChameleonException;
   private native String CHMlistenerGetHeaderBlock(long ListenerHandle) throws ChameleonException;
   private native void CHMlistenerSetHeaderBlock(long ListenerHandle, String Value) throws ChameleonException;
   private native String CHMlistenerGetTrailerBlock(long ListenerHandle) throws ChameleonException;
   private native void CHMlistenerSetTrailerBlock(long ListenerHandle, String Value) throws ChameleonException;
   private native boolean CHMlistenerGetIsListening(long ListenerHandle) throws ChameleonException;
   private native boolean CHMlistenerGetIsValidSocketHandle(long ListenerHandle) throws ChameleonException;
   private native int CHMlistenerGetSocketHandle(long ListenerHandle) throws ChameleonException;
   private native LlpConnection CHMlistenerGetConnection(long ListenerHandle, int ClientIndex) throws ChameleonException;
   private native int CHMlistenerCountOfConnection(long ListenerHandle) throws ChameleonException;
   private native void CHMlistenerStopMessageLoop(long ListenerHandle) throws ChameleonException;static 
   private native void CHMlistenerRunMessageLoop() throws ChameleonException;
   private native void CHMlistenerStartListen(long ListenerHandle, int Port) throws ChameleonException;
   private native String CHMlistenerMakePrintable(long ListenerHandle, String InputString, String NewLine, int MaximumLength) throws ChameleonException;
   private native void CHMlistenerStopListen(long ListenerHandle) throws ChameleonException;
   private native void CHMlistenerCloseAll(long ListenerHandle) throws ChameleonException;
}