package com.interfaceware.chameleon;



/**
 * <p>The listener interface for receiving {@link LlpServer LlpServer} events. A 
 * class that is interested in processing a LlpServer event must implement this 
 * interface (and all the methods it contains). The listener object created from 
 * that class is then registered with a LlpServer object using the LlpServer's 
 * {@link LlpServer#addListener addListener} method. When the LlpServer's state 
 * changes, the relevant method in the listener object is invoked. If you wish the 
 * LlpServer object to stop receiving notifications, use the LlpServer's {@link 
 * LlpServer#removeListener removeListener} method.</p>
 * 
 * @see  LlpServer
 */
public interface LlpServerListener
{
   
   /**
    * <p>Fired when an issue occurs with the server socket itself rather than a 
    * specific client.</p>
    * 
    * @param   ErrorMessage   A message describing the error.
    * @param   Code   A code describing the error.
    */
   public void errorReceived(String ErrorMessage, int Code);
   
   /**
    * <p>Fired when a connection is made to the server by a new client.</p>
    * 
    * <p>This event is useful for logging.</p>
    * 
    * @param   Client   The client object that is connecting.
    */
   public void clientConnected(LlpConnection Client);
   
   /**
    * <p>Fired when a client disconnects from the server, or a previous {@link 
    * LlpConnection#disconnect disconnect} call on a Connection succeeds.</p>
    * 
    * <p>This event is useful for logging.</p>
    * 
    * @param   Client   The client object that is disconnecting.
    */
   public void clientDisconnected(LlpConnection Client);
   
   /**
    * <p>Fired when a networking error occurs with a specific client connection.</p>
    * 
    * <p>The context on when this is received is often determined by a previous 
    * operation, although that is not necessary. The connection will always be closed 
    * once this event is received so there is no need to close the connection within 
    * the application code.</p>
    * 
    * @param   Client   The client that had an error.
    * @param   ErrorMessage   The description of the error.
    * @param   ErrorCode   The code that describes the error.
    */
   public void clientErrorReceived(LlpConnection Client, String ErrorMessage, int ErrorCode);
   
   /**
    * <p>Fired each time we receive a message from a connected HL7 client.</p>
    * 
    * <p>Typically, the message should be processed and only then should an HL7 
    * ACKnowledgement message be sent.</p>
    * 
    * <p>Received without any Trailer or Header characters.</p>
    * 
    * @param   Client   The client that has sent the message.
    * @param   Message   The HL7 message (in string format) that has been sent.
    */
   public void messageReceived(LlpConnection Client, String Message);
   
   /**
    * <p>Fired when data has been received that is not within the header and trailer 
    * block envelope.</p>
    * 
    * <p>This is for dealing with an incorrect LLP implementation. The data is 
    * provided to the user's application through this event. This allows it to be 
    * logged since this is likely to be a very significant error with the 
    * counterparty's LLP implementation.</p>
    * 
    * @param   Client   The client that has sent the data.
    * @param   Data   Data in string format that has been sent.
    */
   public void dataIgnored(LlpConnection Client, String Data);
   
}