package com.interfaceware.chameleon;

/**
* This class handles Table objects.
* This class gives the interface to the basic data container
* in the Chameleon framework: a nested in-memory table. 
**/
public class Table
{
   private boolean m_TableReleased;
/**
* Construct a Table object.
**/
   public Table() throws ChameleonException
   {
      m_TableHandle = CHMtableCreate();
      CHMtableAddRef(m_TableHandle);
      m_TableReleased = false;
   }

/**
* Construct a Table object with the specified table handle.
* This is used in the internal implementation of Chameleon and should
* not be invoked directly.
**/
   public Table(long Handle) throws ChameleonException
   {
      m_TableHandle = Handle;
      m_TableReleased = true;
   }

/**
* Construct a Table object with the specified table handle.
* This is used in the internal implementation of Chameleon and should
* not be invoked directly.
**/
// This constructor does not increment the reference to the
//underlying table object.
   public Table(long Handle, boolean IsOwner) throws ChameleonException
   {
      m_TableHandle = Handle;
      // We do not increment the reference count of the underlying
      // table object.  
      m_TableReleased = true;
   }

/**
* Free the Table object handle.
**/
   public void release() throws ChameleonException
   {
      if (!m_TableReleased)
      {
         CHMtableRelease(m_TableHandle);
         m_TableReleased = true;
      }
   }

/**
* Finalizer that invokes the release method.
**/
   protected void finalize() throws ChameleonException
   {
      release();
   }

/**
* Get the table name.
**/
   public String tableName() throws ChameleonException
   {
      return CHMtableName(m_TableHandle);
   }

/**
* Get the number of table columns.  
* Usage of this method with {@link #columnName(int) columnName} offers an easy way 
* to iterate through the columns in the table.
**/
   public int countOfColumn() throws ChameleonException
   {
      return CHMtableCountOfColumn(m_TableHandle);
   }

/**
* Get the number of table rows.
**/
   public int countOfRow() throws ChameleonException
   {
      return CHMtableCountOfRow(m_TableHandle);
   }

/**
* Get the number of sub-tables.
**/
   public int countOfSubTable() throws ChameleonException
   {
      return CHMtableCountOfSubTable(m_TableHandle);
   }

/**
* Get the sub-table with the specified row and table index.
**/
   public Table subTable(int RowIndex, int TableIndex) throws ChameleonException
   {
      long TableHandle = CHMtableSubTable(m_TableHandle, RowIndex, TableIndex);
      return new Table(TableHandle);
   }

/**
* Get the table column name.
* Usage of this method with {@link #countOfColumn() countOfColumn}
* offers an easy way to iterate through the columns in a table.
**/
   public String columnName(int ColumnIndex) throws ChameleonException
   {
      return CHMtableColumnName(m_TableHandle, ColumnIndex);
   }

/**
* Get the column index of the column named <code>ColumnName</code>.
* Returns -1 if there is no column with that name.
*
**/
   public int columnIndex(String ColumnName) throws ChameleonException
   {
      return CHMtableColumnIndex(m_TableHandle, ColumnName);
   }

/**
* Get the column type of the column at the specified column index.
* @see com.interfaceware.chameleon.DataType
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
* 
**/
   public int columnType(int ColumnIndex) throws ChameleonException
   {
      return CHMtableColumnType(m_TableHandle, ColumnIndex);
   }

/**
* Add a row to the table.
**/
   public void addRow() throws ChameleonException
   {
      CHMtableAddRow(m_TableHandle);
   }
/**
* Set the value at <code>ColumnIndex</code> and <code>RowIndex</code> of a String-type
* column.  Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public void setString(int ColumnIndex, int RowIndex, String Value) throws ChameleonException
   {
      CHMtableSetString(m_TableHandle, ColumnIndex, RowIndex, Value);   
   }

/**
* Set the value at <code>ColumnIndex</code> and <code>RowIndex</code> of a Double-type column.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public void setDouble(int ColumnIndex, int RowIndex, Double Value) throws ChameleonException
   {
      CHMtableSetDouble(m_TableHandle, ColumnIndex, RowIndex, Value.doubleValue());
   }
/**
* Set the value at <code>ColumnIndex</code> and <code>RowIndex</code> of a double-type column.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/

   public void setdouble(int ColumnIndex, int RowIndex, double Value) throws ChameleonException
   {
      CHMtableSetDouble(m_TableHandle, ColumnIndex, RowIndex, Value);
   }

/**
* Set the value at <code>ColumnIndex</code> and <code>RowIndex</code> of an integer-type column.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public void setInteger(int ColumnIndex, int RowIndex, int Value) throws ChameleonException
   {
      CHMtableSetInteger(m_TableHandle, ColumnIndex, RowIndex, Value);
   }
   
/**
* Set the value at <code>ColumnIndex</code> and <code>RowIndex</code> of an Integer-type column.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public void setInteger(int ColumnIndex, int RowIndex, Integer Value) throws ChameleonException
   {
      CHMtableSetInteger(m_TableHandle, ColumnIndex, RowIndex, Value.intValue());
   }

/**
* Set the value at <code>ColumnIndex</code> and <code>RowIndex</code> of a date/time-type column.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public void setDate(int ColumnIndex, int RowIndex, ChameleonDateTime Value) throws ChameleonException
   {
      CHMtableSetDateTime(m_TableHandle, ColumnIndex, RowIndex, Value.handle());
   }
/**
* Get the date/time value at <code>ColumnIndex</code> and <code>RowIndex</code>.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public ChameleonDateTime getDate(int ColumnIndex, int RowIndex) throws ChameleonException
   {
      return new ChameleonDateTime(CHMtableGetDateTime(m_TableHandle, ColumnIndex, RowIndex)); 
   }

/**
* Get the Integer value at <code>ColumnIndex</code> and <code>RowIndex</code>.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public Integer getInteger(int ColumnIndex, int RowIndex) throws ChameleonException
   {
      return new Integer(CHMtableGetInteger(m_TableHandle, ColumnIndex, RowIndex));
   }

/**
* 
* Get the String value at <code>ColumnIndex</code> and <code>RowIndex</code>.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public String getString(int ColumnIndex, int RowIndex) throws ChameleonException
   {
      return CHMtableGetString(m_TableHandle, ColumnIndex, RowIndex);
   }

/**
* Get the Double value at <code>ColumnIndex</code> and <code>RowIndex</code>.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public Double getDouble(int ColumnIndex, int RowIndex) throws ChameleonException
   {
      return new Double(CHMtableGetDouble(m_TableHandle, ColumnIndex, RowIndex));
   }

/**
* Get the double value at <code>ColumnIndex</code> and <code>RowIndex</code>.
* Throws an exception if the data type doesn't match.
* @see <a href="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</a>
**/
   public double getdouble(int ColumnIndex, int RowIndex) throws ChameleonException
   {
      return CHMtableGetDouble(m_TableHandle, ColumnIndex, RowIndex);
   }

/**
* Check if the value at <code>ColumnIndex</code> and <code>RowIndex</code> is null.
* @see <a href="http://www.interfaceware.com/manual/is_null.html">http://www.interfaceware.com/manual/is_null.html</a>
**/
   public boolean isNull(int ColumnIndex, int RowIndex) throws ChameleonException
   {
      return CHMtableIsNull(m_TableHandle, ColumnIndex, RowIndex);
   }

/**
* Test the value in the cell at row <code>RowIndex</code> and column <code>ColumnIndex</code> to see if it is null, zero, empty or invalid.
* @see <a href="http://interfaceware.com/manual/is_null.html">http://interfaceware.com/manual/is_null.html</a>
**/
   public int getState(int ColumnIndex, int RowIndex) throws ChameleonException
   {
      return CHMtableState(m_TableHandle, ColumnIndex, RowIndex);
   }


/**
* Convert the table data to a string.
**/
   public final synchronized String toString() 
   {
      try
      {
         return CHMtableDump(m_TableHandle);
      }
      catch (ChameleonException Error)
      {
         Error.printStackTrace();
         return Error.getMessage();
      }
   }

/**
* Get the table handle.  
* This is used in the internal implementation of Chameleon and should not be invoked directly.
**/
   public long handle()
   {
      return m_TableHandle;
   }

/**
* The table handle member variable.
* This is used in the internal implementation of Chameleon and should not be used directly.
**/
   protected long m_TableHandle;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native long CHMtableCreate() throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native void CHMtableAddRef(long TableHandle) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native void CHMtableRelease(long TableHandle) throws ChameleonException;
   /**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native String CHMtableName
   (
      long TableHandle
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native long CHMtableSubTable
   (
      long TableHandle,
      int RowIndex,
      int TableIndex
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native int CHMtableCountOfColumn
   (
      long TableHandle
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native int CHMtableCountOfRow
   (
      long TableHandle
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native int CHMtableCountOfSubTable
   (
      long TableHandle
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native int CHMtableColumnType
   (
      long TableHandle,
      int ColumnIndex
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native String CHMtableColumnName
   (
      long TableHandle,
      int ColumnIndex
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native int CHMtableColumnIndex
   (
      long    TableHandle,
      String ColumnName
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native boolean CHMtableIsNull
   (
      long TableHandle,
      int ColumnIndex,
      int RowIndex
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native int CHMtableState
   (
      long TableHandle,
      int ColumnIndex,
      int RowIndex
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native String CHMtableGetString
   (
      long TableHandle,
      int ColumnIndex,
      int RowInde
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native double CHMtableGetDouble
   (
      long TableHandle,
      int ColumnIndex,
      int RowIndex
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native int CHMtableGetInteger
   (
      long TableHandle,
      int ColumnIndex,
      int RowIndex
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   // returns handle to date time object
   protected native long CHMtableGetDateTime
   (
      long TableHandle,
      int ColumnIndex,
      int RowIndex
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native void CHMtableSetString
   (
      long    TableHandle,
      int    ColumnIndex,
      int    RowIndex,
      String Value
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native void CHMtableSetDouble
   (
      long    TableHandle,
      int    ColumnIndex,
      int    RowIndex,
      double Value
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native void CHMtableSetInteger
   (
      long TableHandle,
      int ColumnIndex,
      int RowIndex,
      int Value
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native void CHMtableSetDateTime
   (
      long TableHandle,
      int ColumnIndex,
      int RowIndex,
      long DateTimeHandleValue
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native void CHMtableAddRow
   (
      long TableHandle
   ) throws ChameleonException;
/**
 * Used in the internal implementation of Chameleon; should not be invoked directly.
 **/
   protected native String CHMtableDump
   (
      long TableHandle
   ) throws ChameleonException;
   static
   {
      System.loadLibrary("chm_java");
   }
}
